%{?scl:%scl_package lld}
%{!?scl:%global pkg_name %{name}}
%global devtoolset_name devtoolset-9

#prevent byte-compilation of python test code
%global __os_install_post %(echo '%{__os_install_post}' | sed -e 's!/usr/lib[^[:space:]]*/brp-scl-python-bytecompile[[:space:]].*$!!g')

#%%global rc_ver 6
%global baserelease 2
%global lld_srcdir lld-%{version}%{?rc_ver:rc%{rc_ver}}.src
%global maj_ver 14
%global min_ver 0
%global patch_ver 5

# Don't include unittests in automatic generation of provides or requires.
%global __provides_exclude_from ^%{_libdir}/lld/.*$
%global __requires_exclude ^libgtest.*$

%bcond_with ld_alternative


Name:		%{?scl_prefix}lld
Version:	%{maj_ver}.%{min_ver}.%{patch_ver}
Release:	%{baserelease}%{?rc_ver:.rc%{rc_ver}}%{?dist}
Summary:	The LLVM Linker

License:	NCSA
URL:		http://llvm.org
Source0:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{version}%{?rc_ver:-rc%{rc_ver}}/%{lld_srcdir}.tar.xz
Source1:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{version}%{?rc_ver:-rc%{rc_ver}}/%{lld_srcdir}.tar.xz.sig
Source2:	tstellar-gpg-key.asc
Source3:	run-lit-tests
Source4:	lit.lld-test.cfg.py

ExcludeArch:	s390x

Patch0:		0001-PATCH-lld-CMake-Check-for-gtest-headers-even-if-lit..patch

# Bundle libunwind header need during build for MachO support
Patch1:		0002-PATCH-lld-Import-compact_unwind_encoding.h-from-libu.patch

BuildRequires:	%{?scl:%{devtoolset_name}-}gcc
BuildRequires:	%{?scl:%{devtoolset_name}-}gcc-c++
BuildRequires:	%{?scl_prefix}cmake
BuildRequires: %{?scl_prefix}llvm-devel = %{version}
BuildRequires:	%{?scl_prefix}llvm-test = %{version}
BuildRequires:	ncurses-devel
BuildRequires:	zlib-devel
BuildRequires:	python3-devel

# For make check:
BuildRequires:	python3-rpm-macros
BuildRequires:	%{?scl_prefix}python3-lit
BuildRequires:	%{?scl_prefix}llvm-googletest = %{version}

%if %{with ld_alternative}
Requires(post): %{_sbindir}/alternatives
Requires(preun): %{_sbindir}/alternatives
%endif

Requires: %{?scl_prefix}lld-libs = %{version}-%{release}

%description
The LLVM project linker.

%package devel
Summary:	Libraries and header files for LLD
Requires: %{?scl_prefix}lld-libs = %{version}-%{release}
# lld tools are referenced in the cmake files, so we need to add lld as a
# dependency.
Requires: %{name}%{?_isa} = %{version}-%{release}

%description devel
This package contains library and header files needed to develop new native
programs that use the LLD infrastructure.

%package libs
Summary:	LLD shared libraries

%description libs
Shared libraries for LLD.

%package test
Summary: LLD regression tests
Requires:      %{name}%{?_isa} = %{version}-%{release}
Requires:      %{?scl_prefix}python-lit
Requires:      %{?scl_prefix}llvm-test(major) = %{maj_ver}
Requires:      %{?scl_prefix}lld-libs = %{version}-%{release}

%description test
LLVM regression tests.

# LLVM libraries and tools no longer set rpath, so we need to enable the scl
# so llvm-config can find libLLVM.so.
%enable_llvmtoolset14_0

%prep

%autosetup -n %{lld_srcdir} -p2



%build

# Disable lto since it causes the COFF/libpath.test lit test to crash.
%global _lto_cflags %{nil}

mkdir %{_target_platform}
cd %{_target_platform}

# We don't set rpath
export LD_LIBRARY_PATH=${PWD}/%{_lib}:${LD_LIBRARY_PATH}


%cmake .. \
	-DLLVM_LINK_LLVM_DYLIB:BOOL=ON \
        -DCMAKE_C_COMPILER=/opt/rh/%{devtoolset_name}/root/usr/bin/gcc\
        -DCMAKE_CXX_COMPILER=/opt/rh/%{devtoolset_name}/root/usr/bin/g++\
	-DLLVM_DYLIB_COMPONENTS="all" \
	-DCMAKE_SKIP_RPATH:BOOL=ON \
	-DPYTHON_EXECUTABLE=%{__python3} \
	-DLLVM_INCLUDE_TESTS=ON \
	-DLLVM_MAIN_SRC_DIR=%{_datadir}/llvm/src \
	-DLLVM_EXTERNAL_LIT=%{_bindir}/lit \
	-DLLVM_LIT_ARGS="-sv \
	--path %{_libdir}/llvm" \
%if 0%{?__isa_bits} == 64
	-DLLVM_LIBDIR_SUFFIX=64
%else
	-DLLVM_LIBDIR_SUFFIX=
%endif

export LD_LIBRARY_PATH=$PWD/%{_lib}:${LD_LIBRARY_PATH}

%make_build

# Build the unittests so we can install them.
%make_build lld-test-depends

%install

%global lit_cfg test/%{_arch}.site.cfg.py
%global lit_unit_cfg test/Unit/%{_arch}.site.cfg.py
%global lit_lld_test_cfg_install_path %{_datadir}/lld/lit.lld-test.cfg.py

# Generate lit config files.  Strip off the last line that initiates the
# test run, so we can customize the configuration.
head -n -1 %{_target_platform}/test/lit.site.cfg.py >> %{lit_cfg}
head -n -1 %{_target_platform}/test/Unit/lit.site.cfg.py >> %{lit_unit_cfg}

# Patch lit config files to load custom config:
for f in %{lit_cfg} %{lit_unit_cfg}; do
  echo "lit_config.load_config(config, '%{lit_lld_test_cfg_install_path}')" >> $f
done

# Install test files
install -d %{buildroot}%{_datadir}/lld/src
cp %{SOURCE4} %{buildroot}%{_datadir}/lld/

# The various tar options are there to make sur the archive is the same on 32 and 64 bit arch, i.e.
# the archive creation is reproducible. Move arch-specific content out of the tarball
mv %{lit_cfg} %{buildroot}%{_datadir}/lld/src/%{_arch}.site.cfg.py
mv %{lit_unit_cfg} %{buildroot}%{_datadir}/lld/src/%{_arch}.Unit.site.cfg.py
tar --mtime='UTC 2020-01-01' -c test/ | gzip -n > %{buildroot}%{_datadir}/lld/src/test.tar.gz

install -d %{buildroot}%{_libexecdir}/tests/lld
install -m 0755 %{SOURCE3} %{buildroot}%{_libexecdir}/tests/lld

# Install unit test binaries
install -d %{buildroot}%{_libdir}/lld/

rm -rf `find %{buildroot}%{_libdir}/lld/ -iname '*make*'`

# Install gtest libraries
cp %{_target_platform}/%{_lib}/libgtest*so* %{buildroot}%{_libdir}/lld/

# Install libraries and binaries
cd %{_target_platform}
%make_install

# This is generated by Patch1 during build and (probably) must be removed afterward
rm %{buildroot}%{_includedir}/mach-o/compact_unwind_encoding.h

%if %{with ld_alternative}
# Required when using update-alternatives:
# https://docs.fedoraproject.org/en-US/packaging-guidelines/Alternatives/
touch %{buildroot}%{_bindir}/ld

%post
%{_sbindir}/update-alternatives --install %{_bindir}/ld ld %{_bindir}/ld.lld 1

%postun
if [ $1 -eq 0 ] ; then
  %{_sbindir}/update-alternatives --remove ld %{_bindir}/ld.lld
fi
%endif

%check

%ifnarch ppc64 ppc64le
make -C %{_target_platform} %{?_smp_mflags} check-lld
%endif

%post libs -p /sbin/ldconfig
%postun libs -p /sbin/ldconfig

%files
%license LICENSE.TXT
%if %{with ld_alternative}
%ghost %{_bindir}/ld
%endif
%{_bindir}/lld*
%{_bindir}/ld.lld
%{_bindir}/ld64.lld
%{_bindir}/wasm-ld

%files devel
%{_includedir}/lld
%{_libdir}/liblld*.so
%{_libdir}/cmake/lld/

%files libs
%{_libdir}/liblld*.so.*

%files test
%{_libexecdir}/tests/lld/
%{_libdir}/lld/
%{_datadir}/lld/src/test.tar.gz
%{_datadir}/lld/src/%{_arch}.site.cfg.py
%{_datadir}/lld/src/%{_arch}.Unit.site.cfg.py
%{_datadir}/lld/lit.lld-test.cfg.py

%changelog
* Wed Feb 09 2022 Serge Guelton - 13.0.1-2
- Activate testing

* Thu Feb 03 2022 Tom Stellard <tstellar@redhat.com> - 13.0.1-1
- 13.0.1 Release

* Fri Oct 29 2021 Tom Stellard <tstellar@redhat.com> - 13.0.0-1
- 13.0.0 Release

* Tue Jul 20 2021 sguelton@redhat.com - 12.0.1-1
- LLVM 12.0.1

* Wed Jun 30 2021 sguelton@redhat.com - 12.0.0-1
- llvm 12.0.0

* Fri Mar 05 2021 sguelton@redhat.com - 11.0.1-2
- Fix rhbz#1926974

* Mon Jan 25 2021 sguelton@redhat.com - 11.0.1-1
- 11.0.1 Release

* Wed Jul 22 2020 <sguelton@redhat.com> - 10.0.1-1
- 10.0.1 Release

* Mon Jul 20 2020 sguelton@redhat.com - 10.0.0-3
- tar doesn't support --sort on rhel7

* Mon Jul 20 2020 sguelton@redhat.com - 10.0.0-2
- Restore chrpath calls
- Manage arch independent test package

* Mon Jul 13 2020 sguelton@redhat.com - 10.0.0-1
- 10.0.0 release

* Tue Mar 10 2020 Tom Stellard <tstellar@redhat.com> - 9.0.1-4
- Depend on python2-lit.  There is no python3 in rhel7

* Tue Mar 10 2020 Tom Stellard <tstellar@redhat.com> - 9.0.1-3
- Fix dependency on python3-lit

* Fri Jan 31 2020 Tom Stellard <tstellar@redhat.com> - 9.0.1-2
- Add scl_prefix to lld-test requires

* Tue Jan 28 2020 Tom Stellard <tstellar@redhat.com> - 9.0.1.-1
- 9.0.1 Release

* Wed Aug 28 2019 sguelton@redhat.com - 8.0.1-5
- Remove __cmake hack

* Tue Aug 06 2019 Tom Stellard <tstellar@redhat.com> - 8.0.1-4
- Fix explicit -libs requires in -devel and also add it to the main package

* Tue Aug 06 2019 Tom Stellard <tstellar@redhat.com> - 8.0.1-3
- Add explicit -libs requires to the -devel package

* Fri Aug 2 2019 sguelton@redhat.com - 8.0.1-2
- Update cmake dependency

* Mon Jul 29 2019 sguelton@redhat.com - 8.0.1-1
- 8.0.1 Release

* Fri Jan 11 2019 Tom Stellard <tstellar@redhat.com> - 7.0.1-2
- Make lld-libs an explicit dependency of lld-devel

* Thu Jan 10 2019 Tom Stellard <tstellar@redhat.com> - 7.0.1-1
- 7.0.1 Release

* Wed Jun 27 2018 Tom Stellard <tstellar@redhat.com> - 6.0.1-1
- 6.0.1 Release

* Fri May 11 2018 Tom Stellard <tstellar@redhat.com> - 6.0.1-0.1.rc1
- 6.0.1-rc1 Release

* Thu Mar 08 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-1
- 6.0.0 Release

* Tue Feb 13 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-0.3.rc2
- 6.0.0-rc2 Release

* Thu Feb 08 2018 Fedora Release Engineering <releng@fedoraproject.org> - 6.0.0-0.2.rc1
- Rebuilt for https://fedoraproject.org/wiki/Fedora_28_Mass_Rebuild

* Thu Jan 25 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-0.1.rc1
- 6.0.0-rc1 Release

* Thu Dec 21 2017 Tom Stellard <tstellar@redhat.com> - 5.0.1-1
- 5.0.1 Release

* Mon Sep 11 2017 Tom Stellard <tstellar@redhat.com> - 5.0.0-1
- 5.0.0 Release

* Thu Aug 03 2017 Fedora Release Engineering <releng@fedoraproject.org> - 4.0.1-4
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Binutils_Mass_Rebuild

* Wed Jul 26 2017 Fedora Release Engineering <releng@fedoraproject.org> - 4.0.1-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Mass_Rebuild

* Thu Jul 06 2017 Tom Stellard <tstellar@redhat.com> - 4.0.1-2
- Backport r307092

* Tue Jul 04 2017 Tom Stellard <tstellar@redhat.com> - 4.0.1-1
- 4.0.1 Release

* Tue Jul 04 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-4
- Fix build without llvm-static

* Wed May 31 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-3
- Remove llvm-static dependency

* Mon May 15 2017 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.0.0-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_26_27_Mass_Rebuild

* Tue Mar 14 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-1
- lld 4.0.0 Final Release
